import sqlite3
import cPickle

WSF_DB_MESSAGES = 'messages'
WSF_DB_HIST = 'history'

class WsfSession(object):
    ''' This class holds the connection to the SQLite db and provides and easy
    interface for storing and retrieving HTTP messages for that given browsing
    session '''

    def __init__(self, dbfile):
        self._dbfile = dbfile
        self.mid = 0
        # get connection handler and cursor
        try:
            self._con = sqlite3.connect(self._dbfile, check_same_thread=False)
            self._cur = self._con.cursor()
        except sqlite3.Error, e:
            raise e

        # check if table is created
        self.__chk_table()

        # if it was a loaded file update the mid field
        self.mid = self.__get_mid()+1

    def __del__(self):
        self._cur.close()
        self._con.close()

    def __chk_table(self):
        ''' Checks if the messages table exists other wise it creates it '''
        try:
            q = "CREATE TABLE IF NOT EXISTS %s(mid INT PRIMARY KEY, "\
                "message BLOB)" % WSF_DB_MESSAGES
            self._cur.execute(q)
            q = "CREATE TABLE IF NOT EXISTS %s(mid INT PRIMARY KEY, "\
                "host text, method text, url text, params text, "\
                "status text, size text, modified text, elapsed text"\
                ")" % WSF_DB_HIST
            self._cur.execute(q)
            self._con.commit()
        except sqlite3.Error, e:
            #FIXME sort his out
            print "error __chk_table: %s" % e

    def __get_mid(self):
        mid = -1
        try:
            q = "SELECT * FROM %s ORDER BY mid DESC LIMIT 1" % (WSF_DB_MESSAGES)
            self._cur.execute(q)
            r = self._cur.fetchone()
            if r:
                mid = r[0]
        except sqlite3.Error, e:
            #FIXME yet another
            print "get_mid: %s" % e
        return mid

    def store(self, m):
        ''' Stores a message in the database '''
        try:
            dmp = cPickle.dumps(m, cPickle.HIGHEST_PROTOCOL)
            q = 'INSERT OR REPLACE INTO %s VALUES (?, ?)' % WSF_DB_MESSAGES
            self._cur.execute(q, [m.mid, sqlite3.Binary(dmp)])
            self._con.commit()
        except sqlite3.Error, e:
            # FIXME sorthis out
            print "ERROR store: %s" % e

    def retrieve(self, mid):
        ''' If 'mid' WsfMessage id is found in database returns a tuple in the
        format (request, response) '''
        # FIXME what if it finishes
        m = None
        try:
            q = "SELECT * FROM %s WHERE mid=%s LIMIT 1" % (WSF_DB_MESSAGES, mid)
            self._cur.execute(q)
            r = self._cur.fetchone()
            m = cPickle.loads(str(r[1]))
        except sqlite3.Error, e:
            # FIXME sorthis out
            print "store: %s" % e
        return m

    def store_pres(self, m):
        try:
            q = "INSERT OR REPLACE INTO %s VALUES %s" % (WSF_DB_HIST, \
                m.as_gui())
            self._cur.execute(q)
            self._con.commit()
        except sqlite3.Error, e:
            # FIXME sorthis out
            print "store_pres: %s" % e

    def retrieve_pres(self):
        pres = []
        try:
            q = "SELECT * FROM %s ORDER BY mid ASC" % (WSF_DB_HIST)
            self._cur.execute(q)
            r = self._cur.fetchall()
            if r:
                pres = r
        except sqlite3.Error, e:
            #FIXME yet another
            print "retrieve_pres: %s" % e
        return pres

    def changes(self):
        changes = False
        try:
            q = "SELECT * FROM %s ORDER BY mid DESC LIMIT 1" % (WSF_DB_MESSAGES)
            self._cur.execute(q)
            r = self._cur.fetchone()
            if r:
                changes = True
        except sqlite3.Error, e:
            #FIXME yet another
            print "changes: %s" % e
        return changes

